from pathlib import Path

from pydantic import BaseModel
from pydantic.dataclasses import dataclass
from yaml import safe_load


@dataclass
class Reference:
    title: str
    url: str


class CodeMeta(BaseModel):
    questions: list[str] = []
    excercises: list[str] = []
    references: list[Reference] = []

    @classmethod
    def from_yaml(cls, path: str):
        dict_ = safe_load(Path(path).read_text())
        return cls.parse_obj(dict_)



"""Print today's date and a calendar for current month."""

from calendar import TextCalendar
from datetime import date

t = date.today()
print("Today is", t)
print()  # prints empty line
TextCalendar().prmonth(t.year, t.month)



"""Turn a string into ASCII art using a font style."""

from random import choice

from art import ASCII_FONTS, text2art  # type: ignore

# Print available font style names
print("Available fonts:", ", ".join(ASCII_FONTS))

# Print sample ASCII art
font = choice(ASCII_FONTS)
text = "Python 3"
print("\nSample font:", font)
print("Sample text:", text)
result = text2art("Python 3", font=font, space=1)
print("ASCII art:")
print(result)
print("Characters printed:", len(result))



import ast
import subprocess
import sys
from dataclasses import dataclass
from pathlib import Path

import yaml
from rich.console import Console
from rich.markdown import Markdown
from rich.syntax import Syntax

from lamr.yaml_model import CodeMeta

__all__ = ["CodeFile", "MarkdownFile", "print_md", "ls", "here"]


def print_md(text: str, header: str = "", use_pager=False):
    console = Console(width=80)
    if header:
        header = header.title()

        text = f"# {header}\n\n" + text
    md = Markdown(text)
    if use_pager:
        with console.pager():
            console.print(md)
    else:
        console.print(md)


def print_code(filename: str):
    console = Console(width=80)
    syntax = Syntax.from_path(filename)
    console.print(syntax)


def print_code_text(text: str):
    console = Console(width=80)
    syntax = Syntax(text, lexer="py")
    console.print(syntax)


def here() -> Path:
    return Path(__file__).parent


def get_docstring(file_contents: str):
    module = ast.parse(file_contents)
    docstring = ast.get_docstring(module)
    return docstring if docstring else ""


def ls():
    for path in CodeFile("any").path.parent.iterdir():
        if path.is_file() and str(path).endswith(".py"):
            print(path.name + "\t" + get_docstring(path.read_text()))


@dataclass
class File:
    filename: str
    folder: str

    def assert_exists(self):
        if not self.path.exists():
            sys.exit("File not found " + str(self.path))
        return self

    @property
    def path(self):
        return here() / self.folder / self.filename

    @property
    def text(self):
        return self.path.read_text()


@dataclass
class YamlFile(File):
    filename: str
    folder: str = "code"

    @property
    def contents(self):
        return yaml.safe_load(self.text)

    @property
    def meta(self):
        return CodeMeta.from_yaml(self.path)


@dataclass
class CodeFile(File):
    filename: str
    folder: str = "code"

    def run(self):
        subprocess.run(["python", self.path.absolute()])

    def print(self):
        print_code_text(self.text)

    @property
    def yaml_file(self) -> YamlFile:
        return YamlFile(self.filename.replace(".py", ".yml"))

    @property
    def meta(self) -> CodeMeta:
        if self.yaml_file.path.exists():
            return self.yaml_file.meta
        else:
            return CodeMeta()


@dataclass
class MarkdownFile(File):
    filename: str
    folder: str = "topics"

    def print(self, paginate: bool):
        print_md(self.post.content, self.post.get("title", ""), paginate)

    @property
    def post(self):
        import frontmatter  # type: ignore

        return frontmatter.load(self.path)



"""Python course on the command line."""

import sys
from random import choice
from typing import Optional

import typer
from rich import print
from rich.markdown import Markdown
from typer import Typer
from typing_extensions import Annotated

from lamr.file_handlers import CodeFile, MarkdownFile, here, ls, print_md

lamr_app = Typer(
    add_completion=False,
    help="Python course on the command line.",
)


def print_from_root(filename):
    return print_md((here().parent / filename).read_text())


@lamr_app.command()
def about(contributors: bool = False, dev: bool = False):
    """Print a README file."""
    if contributors:
        print("""Contributors: ...""")
    elif dev:
        print_from_root("development.md")
    else:
        print_from_root("README.md")


def list_as_markdown(items) -> Markdown:
    strings = []
    for i, item in enumerate(items):
        strings.append(str(i + 1) + ". " + item)
    return Markdown("\n".join(strings))


def references_as_markdown(references) -> Markdown:
    strings = []
    for i, ref in enumerate(references):
        strings.append(str(i + 1) + ". " + f"{ref.title} URL: <{ref.url}>")
    return Markdown("\n".join(strings))


def bold(text) -> Markdown:
    return Markdown("**" + text.capitalize() + "**")


@lamr_app.command()
def code(
    filename: Annotated[Optional[str], typer.Argument()] = None,
    list_: Annotated[bool, typer.Option("--list")] = False,
    questions: bool = False,
    excercises: bool = False,
    references: bool = False,
    all_: Annotated[bool, typer.Option("--all")] = False,
):
    """Show code example (with questions, excercises or references)."""
    if filename is not None:
        code_file = CodeFile(filename).assert_exists()
        code_file.print()
        meta = code_file.meta
        if meta.questions and (questions or all_):
            print(bold("Review questions"))
            print(list_as_markdown(meta.questions))
            print()
        if meta.excercises and (excercises or all_):
            print(bold("Excercises"))
            print(list_as_markdown(meta.excercises))
            print()
        if meta.references and (references or all_):
            print(bold("References"))
            print(references_as_markdown(meta.references))
    if filename is None or list_:
        ls()


@lamr_app.command()
def run(filename: str):
    """Run code example."""
    CodeFile(filename).assert_exists().run()


# TRY: console.pager() loses color maybe use scrolling by section
#      with "Press any key to continue..." after each section
def show(md_file: str, paginate: bool = False):
    """Show page."""
    MarkdownFile(md_file).print(paginate)


topic_list = [
    "programming",
    "editors",
    "run-your-code",
    "print",
    "value",
    "numbers",
    "strings",
    "bools",
    "expression",
    "variables",
    "input",
    "if-else",
    "lists",
    "for-loop",
    "while-loop",
    "functions",
    "types",
]

tree = dict(variables=["variable-assignment.md"])


def print_list(header: str, items: list[str]):
    eol = "\n  "
    print(header + ":" + eol, eol.join(items), sep="")


@lamr_app.command()
def learn(
    topic: Annotated[Optional[str], typer.Argument()] = None,
    random: bool = False,
    tentative_topics: bool = False,
):
    """Learn or review a topic."""
    if topic in tree.keys():
        for md_file in tree[topic]:
            show(md_file)
        sys.exit(0)
    if random:
        learn(choice(list(tree.keys())))  # mypy wants it this way
    print_list("Available topics", list(tree.keys()))  # mypy wants it this way
    if tentative_topics:
        print_list("Tentative list of topics", topic_list)
